#include <nds.h>
#include <maxmod9.h>
#include <nf_lib.h>
#include <stdio.h>
#include "soundbank.h"
#include "multiplayermenu.h"
#include "soundbank_bin.h"

volatile int frame = 0;

void Multiplayer()
{
    u8 rfi, gfi, bfi;
    s16 cRfi, cGfi, cBfi;
    u8 speedfi = 0;
    u16 nfi = 0;

    for (int i = 0; i < 125; i++) { // Loop from 0 to 255 for smooth transition
        for (nfi = 0; nfi < 256; nfi++) {
            NF_BgGetPalColor(0, 3, nfi, &rfi, &gfi, &bfi);
            cRfi = rfi;
            cGfi = gfi;
            cBfi = bfi;
            cRfi--;
            if (cRfi < 0) cRfi = 0;
            cGfi--;
            if (cGfi < 0) cGfi = 0;
            cBfi--;
            if (cBfi < 0) cBfi = 0;
            NF_BgEditPalColor(0, 3, nfi, cRfi, cGfi, cBfi);
        }
        NF_BgUpdatePalette(0, 3); // Update the palette

        speedfi++;
        if (speedfi > 1) { // Adjust this value to control the speed of the transition
            speedfi = 0;
        }

        NF_SpriteOamSet(0);
        NF_SpriteOamSet(1);

        swiWaitForVBlank(); // Wait for vertical blank

        oamUpdate(&oamMain);
        oamUpdate(&oamSub);
    }
    MultiplayerMatch();
}

void Initialize()
{
    NF_SetRootFolder("NITROFS");
    NF_Set2D(0, 0);
    NF_Set2D(1, 0);
    NF_InitSpriteSys(1);
    NF_SpriteOamSet(1);

    NF_InitTiledBgBuffers();
    NF_InitTiledBgSys(0);
    NF_InitTiledBgSys(1);

    NF_InitSpriteBuffers();
    NF_InitSpriteSys(0);

    NF_LoadSpriteGfx("ui/birdekek", 0, 64, 64);
    NF_LoadSpritePal("ui/birdekek", 0);

    NF_LoadTiledBg("maps/mainscreen", "main_bg", 256, 256);
    NF_LoadTiledBg("maps/bottomscreen", "bottom_bg", 256, 256);
    NF_LoadTiledBg("maps/msc", "bottom_select", 256, 256);
    NF_LoadTiledBg("maps/mscoff", "bottom_select_two", 256, 256);
    NF_LoadTiledBg("maps/mscon", "bottom_select_one", 256, 256);

    NF_VramSpriteGfx(0, 0, 0, false);
    NF_VramSpritePal(0, 0, 0);
}

void Vblank()
{
    frame++;
}

int main(int argc, char** argv)
{
    mmInitDefaultMem((mm_addr)soundbank_bin);

    irqSet(IRQ_VBLANK, Vblank);

    consoleDemoInit();

    Initialize();

    consoleClear();

    NF_CreateSprite(0, 0, 0, 0, 96, 32);

    mmLoadEffect(SFX_BIRDEKEK);
    mmLoadEffect(SFX_SELECT);

    mmEffect(SFX_BIRDEKEK);

    for (int i = 0; i < 120; i++)
    {
        NF_SpriteOamSet(0);
        swiWaitForVBlank();
        oamUpdate(&oamMain);
    }

    setBrightness(0, 0);

    NF_DeleteSprite(0, 0);

    mmLoad(MOD_FINALTWIST);
    mmStart(MOD_FINALTWIST, MM_PLAY_LOOP);

    NF_CreateTiledBg(0, 3, "main_bg");
    NF_CreateTiledBg(1, 3, "bottom_bg");
    NF_CreateTiledBg(1, 1, "bottom_select");

    u8 r, g, b;
    s16 cR, cG, cB;
    u8 speed = 0;
    u16 n = 0;

    int selection = 0;
    int keys;

    NF_FreeSpriteGfx(0, 0);
    NF_UnloadSpritePal(0);

    speed = 0;

    for (int i = 0; i < 8; i++) { // Loop from 0 to 255 for smooth transition
        for (n = 0; n < 256; n++) {
            NF_BgGetPalColor(0, 3, n, &r, &g, &b);
            cR = r;
            cG = g;
            cB = b;

            cR += 25;
            cG += 25;
            cB += 25;
            // Decrease the color components
            cR--;
            if (cR < 0) cR = 0;
            cG--;
            if (cG < 0) cG = 0;
            cB--;
            if (cB < 0) cB = 0;

            // Update palette color
            NF_BgEditPalColor(0, 3, n, cR, cG, cB);
        }
        NF_BgUpdatePalette(0, 3); // Update the palette

        speed++;
        if (speed > 1) { // Adjust this value to control the speed of the transition
            speed = 0;
        }

        NF_SpriteOamSet(0);
        NF_SpriteOamSet(1);

        swiWaitForVBlank(); // Wait for vertical blank

        oamUpdate(&oamMain);
        oamUpdate(&oamSub);
    }

    while (1)
    {
        scanKeys();

        keys = keysDown();

        if (keys & KEY_DOWN) {
            if (selection < 2) {
                selection++;
                mmEffect(SFX_SELECT);
            }
        }

        if (keys & KEY_UP) {
            if (selection > 1) {
                selection--;
                mmEffect(SFX_SELECT);
            }
        }

        if (selection == 1) {
            NF_DeleteTiledBg(1, 1);
            NF_CreateTiledBg(1, 1, "bottom_select_one");
        }

        if (selection == 2) {
            NF_DeleteTiledBg(1, 1);
            NF_CreateTiledBg(1, 1, "bottom_select_two");
        }

        if(keys & KEY_A & (selection == 1)) {
            Multiplayer();
            return 0;
        }

        swiWaitForVBlank();
    }

    return 0;
}